@@ -7,7 +7,7 @@ class ScpiTypeBase(object):
     """Base class for SCPI types"""
     __metaclass__ = abc.ABCMeta
 
-    _encoding: str = 'ascii'
+    _encoding: str = 'utf-8'
     _newline: bytes = b'\r\n'
 
     @abc.abstractmethod
