@@ -1,6 +1,6 @@
 import io
 from typing import *
-from typing import IO
+from typing import BinaryIO
 from .scpi_type_base import ScpiTypeBase
 
 
@@ -26,16 +26,16 @@ class ScpiChars(ScpiTypeBase):
 
     def as_bytes(self) -> bytes:
         """Returns the char data as a Python bytes-object"""
-        return self._data.encode('ascii')
+        return self._data.encode(self._encoding)
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         """Composes the object into a SCPI string to be used as a parameter"""
-        return self._data
+        return self._data.encode(self._encoding)
 
     @classmethod
-    def parse(cls, transport: IO) -> 'ScpiChars':
+    def parse(cls, transport: BinaryIO) -> 'ScpiChars':
         """Parses stream data from the device and creates a `ScpiChars` object"""
-        s = transport.readline()
+        s = transport.readline().rstrip(cls._newline).decode(cls._encoding)
         return ScpiChars(s)
 
 
@@ -92,21 +92,21 @@ class ScpiNumber(ScpiTypeBase):
         """Returns the base of the number as integer"""
         return self._base
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         """Composes the object into a SCPI string to be used as a parameter"""
         if type(self._value) == int:
             # Compose into NR1 representation
-            return f'{self._prefix}{self._value:{self._format}}{(" " + self._unit) if self._unit is not None else ""}'
+            return f'{self._prefix}{self._value:{self._format}}{(" " + self._unit) if self._unit is not None else ""}'.encode(self._encoding)
         elif type(self._value) == float:
             # Compose into NR2/3 representation
-            return f'{self._value:.15g}{(" " + self._unit) if self._unit is not None else ""}'
+            return f'{self._value:.15g}{(" " + self._unit) if self._unit is not None else ""}'.encode(self._encoding)
         else:
             raise TypeError
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiNumber` object"""
-        data = transport.readline()
+        data = transport.readline().rstrip(cls._newline).decode(cls._encoding)
         substrs = data.split(' ')
         base = 10
         prefix = substrs[0][:2] if substrs[0].startswith('#') else None
@@ -162,16 +162,16 @@ class ScpiNumberArray(ScpiTypeBase):
         """Returns the numeric data as a Python list of floats"""
         return [x.as_float() for x in self._numbers]
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         """Composes the object into a SCPI string to be used as a parameter"""
-        return ','.join([x.compose() for x in self._numbers])
+        return b','.join([x.compose() for x in self._numbers])
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiNumberArray` object"""
-        data = transport.readline()
-        substrs = data.split(',')
-        numbers = [ScpiNumber.parse(io.StringIO(substr)) for substr in substrs]
+        data = transport.readline().rstrip(cls._newline)
+        substrs = data.split(b',')
+        numbers = [ScpiNumber.parse(io.BytesIO(substr)) for substr in substrs]
         return ScpiNumberArray(numbers)
 
 
@@ -203,29 +203,29 @@ class ScpiString(ScpiTypeBase):
 
     def as_bytes(self) -> bytes:
         """Returns the char data as a Python bytes-object"""
-        return self._data.encode('ascii')
+        return self._data.encode(self._encoding)
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         """Composes the object into a SCPI string to be used as a parameter"""
-        return f'"{self._data}"'
+        return f'"{self._data}"'.encode(self._encoding)
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiString` object"""
         quotes = 0
-        string = ""
+        string = b""
         while 1:
-            chunk = transport.readline()
-            quotes += chunk.count('"')
-            string += chunk + '\r\n'
+            chunk = transport.readline().rstrip(cls._newline)
+            quotes += chunk.count(b'"')
+            string += chunk + b'\r\n'
 
-            if (quotes % 2 == 0) and (string != ''):
+            if (quotes % 2 == 0) and (string != b''):
                 break
 
-        string = string.strip('\r\n\t"')
-        string = string.replace('""', '"')
+        string = string.strip(b'\r\n\t"')
+        string = string.replace(b'""', b'"')
 
-        return ScpiString(string)
+        return ScpiString(string.decode(cls._encoding))
 
 
 class ScpiArbBlock(ScpiTypeBase):
@@ -249,13 +249,13 @@ class ScpiArbBlock(ScpiTypeBase):
     def as_bytes(self) -> bytes:
         return self._data
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         length = f'{len(self._data)}'
         head = f'{len(length)}'
-        return '#' + head + length + self._data.decode('utf8')
+        return ('#' + head + length).encode(self._encoding) + self._data
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiArbBlock` object"""
         head = transport.read(2)
 
@@ -289,18 +289,19 @@ class ScpiArbStream(ScpiTypeBase, io.RawIOBase):
     Note that this class is derived from io.RawIOBase and can be used for stream processing
     """
 
-    def __init__(self, transport: IO, length: int) -> None:
+    def __init__(self, stream: BinaryIO, length: int, strip: bool = False) -> None:
         """Initialize object from a `ScpiTransportBase` object and length integer"""
         super().__init__()
-        self._transport = transport
+        self._stream = stream
         self._length = length
         self._remaining = length
+        self._strip = strip
 
     def __str__(self) -> str:
         return f'<stream with {self._length} bytes of binary data>'
 
     def fileno(self) -> int:
-        return self._transport.fileno()
+        return self._stream.fileno()
 
     def close(self) -> None:
         pass
@@ -309,10 +310,10 @@ class ScpiArbStream(ScpiTypeBase, io.RawIOBase):
         return False
 
     def isatty(self) -> bool:
-        return self._transport.isatty()
+        return self._stream.isatty()
 
     def readable(self) -> bool:
-        return self._transport.readable()
+        return self._stream.readable()
 
     def readline(self, __size: Optional[int] = -1) -> bytes:
         raise NotImplementedError()
@@ -321,7 +322,7 @@ class ScpiArbStream(ScpiTypeBase, io.RawIOBase):
         raise NotImplementedError()
 
     def seekable(self) -> bool:
-        return self._transport.seekable()
+        return self._stream.seekable()
 
     def writable(self) -> bool:
         return False
@@ -330,23 +331,27 @@ class ScpiArbStream(ScpiTypeBase, io.RawIOBase):
         raise NotImplementedError()
 
     def read(self, __size: int = -1) -> Optional[bytes]:
-        data = self._transport.read(__size)
+        data = self._stream.read(__size)
         self._remaining -= len(data)
 
-        if self._remaining == 0:
+        if self._strip and (self._remaining == 0):
             # eat newline
-            self._transport.readline()
+            self._stream.readline()
 
         return data
 
     def readall(self) -> bytes:
         return self.read(self._remaining)
 
-    def compose(self) -> str:
-        raise NotImplementedError()
+    def compose(self) -> bytes:
+        # Write all data in one go
+        length = f'{self._length}'
+        head = f'{len(length)}'
+        data = self._stream.read(self._length)
+        return ('#' + head + length).encode(self._encoding) + data
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiArbStream` object"""
         head = transport.read(2)
 
@@ -362,7 +367,7 @@ class ScpiArbStream(ScpiTypeBase, io.RawIOBase):
         # read header
         length = int(transport.read(num))
 
-        return ScpiArbStream(transport, length)
+        return ScpiArbStream(transport, length, strip=True)
 
 
 class ScpiBool(ScpiTypeBase):
@@ -388,17 +393,17 @@ class ScpiBool(ScpiTypeBase):
         else:
             return False
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         """Composes the object into a SCPI string to be used as a parameter"""
         if self._onoff:
-            return "ON"
+            return "ON".encode(self._encoding)
         else:
-            return "OFF"
+            return "OFF".encode(self._encoding)
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiBool` object"""
-        data = transport.readline()
+        data = transport.readline().rstrip(cls._newline)
         if int(data) == 0:
             return ScpiBool(False)
         else:
@@ -424,15 +429,15 @@ class ScpiNumList(ScpiTypeBase):
         """Returns the object as a Python list-object"""
         return self._numlist
 
-    def compose(self) -> str:
+    def compose(self) -> bytes:
         """Composes the numeric list into a SCPI string to be used as a parameter"""
-        return '(' + ','.join(map(str, self._numlist)) + ')'
+        return ('(' + ','.join(map(str, self._numlist)) + ')').encode(self._encoding)
 
     @classmethod
-    def parse(cls, transport: IO):
+    def parse(cls, transport: BinaryIO):
         """Parses stream data from the device and creates a `ScpiNumList` object"""
-        data = transport.readline()
-        substrs = data.strip('()').split(',')
+        data = transport.readline().rstrip(cls._newline)
+        substrs = data.strip(b'()').split(b',')
         return ScpiNumList([int(e) for e in substrs])
 
 
@@ -469,8 +474,8 @@ class ScpiEvent(ScpiTypeBase):
         raise NotImplementedError
 
     @classmethod
-    def parse(cls, transport: IO):
-        data = transport.readline()
+    def parse(cls, transport: BinaryIO):
+        data = transport.readline().rstrip(cls._newline).decode(cls._encoding)
         substrs = data.split(',')
         event = substrs[1].strip('"').split(';')
         code = int(substrs[0])
